#!/bin/bash

# shellcheck shell=bash

main() {

  ## arbitraryly pick dist/js to generate the go dir name
  local tgz=$(
    cd dist/js
    ls -1
  )
  local go_dir=$(echo $tgz | awk -F'@' '{print $1}' | tr -d '-')

  clean_js "$go_dir"
  clean_python "$go_dir"
  # clean_dotnet_tgz
  # clean_java_tgz

  return 0
}

remove_junk() {
  local go_dir="$1"

  rm -rf package/node_modules package/.vscode package/.mergify.yml package/.node-version package/eslint.config.mjs "package/$go_dir"

  return 0
}

clean_js() {
  local go_dir="$1"

  echo "====> dist/js:"
  (
    cd dist/js
    local tgz=$(ls -1)

    tar -xzf "$tgz"
    remove_junk "$go_dir"
    echo "$tgz"
    tar -cvzf "$tgz" package
    echo
    rm -rf package
  )

  return 0
}

clean_python() {
  local go_dir="$1"

  echo "====> dist/python:"
  clean_python_tar_gz "$go_dir"
  clean_python_whl "$go_dir"

  return 0
}

clean_python_whl() {
  local go_dir="$1"

  (
    cd dist/python

    local whl=$(ls -1 *.whl)
    local dist_info=$(echo "$whl" | sed -e 's,-py3-none-any.whl,.dist-info,')
    local jsii_tgz=$(echo "$whl" | sed -e 's,-,@,' -e 's,.whl,.jsii.tgz,' -e 's,-py3-none-any,,' | tr '_' '-')

    tar -xzf "$whl"
    local extract_dir=$(ls -1 | grep -v .tar.gz | grep -v .whl | grep -v .dist-info)
    local jsii_dir="/_jsii"

    (
      cd "$extract_dir/$jsii_dir"

      tar -xzf "$jsii_tgz"
      remove_junk "$go_dir"
      rm -f "$jsii_tgz"

      echo "$jsii_tgz"
      tar -cvzf "$jsii_tgz" package/
      rm -rf package/
      echo
    )

    echo "$wnl"
    tar -cvzf "$whl" "$extract_dir" "$dist_info"
    echo

    rm -rf "$extract_dir" "$dist_info"
  )

  return 0
}

clean_python_tar_gz() {
  local go_dir="$1"

  (
    cd dist/python

    local tar_gz=$(ls -1 *.tar.gz)
    local jsii_tgz=$(echo "$tar_gz" | sed -e 's,-,@,' -e 's,.tar.gz,.jsii.tgz,' | tr '_' '-')
    local pkg_dir=$(echo "$tar_gz" | cut -d- -f 1)
    local jsii_dir="src/$pkg_dir/_jsii"
    local path_jsii_tgz="$jsii_dir/$jsii_tgz"

    tar -xzf "$tar_gz"
    local extract_dir=$(ls -1 | grep -v .tar.gz | grep -v .whl)

    (
      cd "$extract_dir/$jsii_dir"

      tar -xzf "$jsii_tgz"
      remove_junk "$go_dir"
      rm -f "$path_jsii_tgz"

      echo "$jsii_tgz"
      tar -cvzf "$jsii_tgz" package/
      rm -rf package/
      echo
    )

    echo "$tar_gz"
    tar -cvzf "$tar_gz" "$extract_dir"
    echo

    rm -rf "$extract_dir"
  )

  return 0
}

main "$@"
